import os
import sys

from bn import uniform_path
from dreamweavertemplate import DreamweaverTemplateInstance
from subprocess import Popen, PIPE

#
# Helpers
#

def run(cmd, surpress_stdout=False, surpress_stderr=False, cwd=None):
    """\
    Execute a command given by the list of parts ``cmd`` in the directory
    ``cwd``.
    """
    stdout = PIPE
    stderr = PIPE
    if not surpress_stderr and hasattr(sys.stderr, 'fileno'):
        stderr = sys.stderr
    if not surpress_stdout and hasattr(sys.stdout, 'fileno'):
        stdout = sys.stdout
    process = Popen(cmd, stdout=stdout, stderr=stderr, cwd=cwd)
    stdout, stderr = process.communicate()
    retval = process.wait()
    if retval:
        print 'FAILED:', ('  '.join(cmd)),
        if cwd:
            print 'in ', cwd,
        print
        if stderr:
            print stderr
        elif stdout:
            print stdout

def cp(src, dst):
    """\
    Copy a file from ``src`` to ``dst``
    """
    src_fp = open(src, 'rb')
    dst_fp = open(dst, 'wb')
    dst_fp.write(src_fp.read())
    dst_fp.close()
    src_fp.close()

#
# Basic commands
#

def remove_dist(path):
    """\
    Remove the ``build`` and ``dist`` directories from the package whose
    package root is at ``path``.
    """
    dist_path = uniform_path(os.path.join(path, 'trunk', 'dist'))
    build_path = uniform_path(os.path.join(path, 'trunk', 'build'))
    if os.path.exists(dist_path):
        print "Removing %s"%dist_path
        run(['rm', '-r', dist_path])
    if os.path.exists(build_path):
        print "Removing %s"%build_path
        run(['rm', '-r', build_path])

def status(path):
    run(['hg', 'st'], cwd=uniform_path(path))

def update(path):
    run(['hg', 'update'], cwd=path)

def pull(path):
    """XXX pexpect needed??"""
    run(['hg', 'pull'], cwd=uniform_path(path))

def build_dist(
    path,
    third_party_eggs_path,
    python='python2.6',
    setuptools_version='setuptools-0.6c11-py2.6.egg',
):
    trunk_path = uniform_path(os.path.join(path, 'trunk'))
    if os.path.exists(trunk_path):
        if not os.path.exists(
            os.path.join(trunk_path, setuptools_version) 
        ):
            print "Copying %s to %r"%(
                os.path.join(third_party_eggs_path, setuptools_version), 
                trunk_path
            )
            cp(
                os.path.join(third_party_eggs_path, setuptools_version), 
                os.path.join(trunk_path, setuptools_version)
            )
        print "Building in %s"%trunk_path
        run([python, 'setup.py', 'sdist'], cwd=trunk_path)

def scan_for_files(base_path, ext=None):
    found = []
    for dirpath, dirname, filenames in os.walk(base_path):
        for filename in filenames:
            if ext is None or filename.lower().endswith('.'+ext):
                found.append(uniform_path(os.path.join(dirpath, filename)))
    return found

def copy_files_to(found, dst):
    for path in found:
        if not path == dst:
            src_filename = uniform_path(path).split('/')[-1]
            dst_filename = os.path.join(dst, src_filename)
            print "Copying %s to %s"%(path, dst_filename)
            cp(path, dst_filename)

def prepare_sources(src, dst):
    copy_files_to(scan_for_files(src, 'tar.gz'), dst)

def test_dist(
    requirement, 
    path, 
    eggs_path, 
    third_party_eggs_path,
    python='python2.6',
    test_requirement=None,
):
    """
    name
        The name of the package (ie its requiement)
    path
        The path to the source dir NOT USED
    source
        The path conating the virtualenv file and setuptools egg
    third_party_eggs_path
        The path to other eggs
    """
    test_path = os.path.join(path, 'trunk', 'dist')
    if not os.path.exists(test_path):
        raise Exception('No test path %s'%test_path)
    print "Creating virtual environment in %s"%test_path
    run(
        [
            python, 
            os.path.join(third_party_eggs_path, 'virtualenv.py'), 
            'env'
        ],
        cwd=test_path
    )
    print "Installing eggs"
    run(
        [
            'env/bin/easy_install', 
            '-f', "%s %s %s"%(
                '.',
                eggs_path.replace(' ', '\\ '), 
                third_party_eggs_path.replace(' ', '\\ '), 
            ),
            '--allow-hosts', '--None--',
            requirement,
        ],
        cwd=test_path
    )
    if test_requirement is not None:
        run(
            [
                'env/bin/easy_install', 
                '-f', "%s %s %s"%(
                    eggs_path.replace(' ', '\\ '), 
                    third_party_eggs_path.replace(' ', '\\ '),
                    '.',
                ),
                '--allow-hosts', '--None--',
                test_requirement,
            ],
            cwd=test_path
        )
    print "Extracting source"
    cur_file = ''
    for file in os.listdir(test_path):
        if file.endswith('.tar.gz'):
            if file > cur_file:
                cur_file = file
    if not cur_file:
        print 'Could not find the distribution file'
    else:
        run(['tar', 'zxfv', cur_file], cwd=test_path)
        cwd = os.path.join(test_path, cur_file[:-len('.tar.gz')], 'test')
        print "Running tests in %r"%cwd
        run(
            [
                '../../env/bin/python', 
                'doc.py',
            ], 
            cwd=cwd,
        )

#
# Compound handlers
#


def handle_one(
    path,
    requirement,
    eggs_path,
    third_party_eggs_path,
    python='python2.6',
    setuptools_version='setuptools-0.6c11-py2.6.egg',
    test_requirement=None,
):
    remove_dist(path)
    build_dist(
        path,
        third_party_eggs_path=third_party_eggs_path,
        python=python,
        setuptools_version=setuptools_version,
    )
    test_dist(
        requirement,
        path, 
        eggs_path=eggs_path,
        third_party_eggs_path=third_party_eggs_path,
        python=python, 
        test_requirement=requirement,
    )

#
# Documentation Handlers
#

def get_pkg_info(name, path):
    fp = open(
        os.path.join(path, 'trunk', '%s.egg-info'%name, 'PKG-INFO'), 
        'rb',
    )
    data = fp.read()
    fp.close()
    pkg_info = {}
    last = None
    for line in data.split('\n'):
        if line.startswith('        '):
            pkg_info[last][-1] += '\n'+line[8:]
        else:
            parts = line.split(': ')
            last = parts[0]
            if not pkg_info.has_key(last):
                pkg_info[last] = []
            value = ': '.join(parts[1:])
            if value.startswith('Development Status :: '):
                pkg_info['Status'] = [value.split('-')[-1].strip()]
            pkg_info[last].append(value)
    return pkg_info

def build_html_docs(name, path):
    run(
        ['make', 'html'],
        cwd=os.path.join(path, 'trunk', 'doc')
    )

def copy_html_docs(docs_path, name, path, version):
    dir = os.path.join(docs_path, name.lower())
    if not os.path.exists(dir):
        os.mkdir(dir)
    dst = os.path.join(dir, version)
    if dst == '/':
        raise Exeception('Problem - this would remove your root filesystem')
    if os.path.exists(dst):
        print "REMOVING %s"%dst
        run(['rm', '-r', dst])
    run(
        [
            'cp',
            '-pr',
            os.path.join(path, 'trunk', 'doc', 'build', 'html'),
            dst,
        ],
    )

def package_index_page(eggs_path, package_docs_path, template_path, pkg_info):
    """\
    docs_path
        the directory containing the docs for all the modules
    dir
        The destination directory for the partcilular package's docs
    pkg_info
        Package Info
    """
    name = pkg_info['Name'][0]
    doc_versions = []
    for filename in os.listdir(package_docs_path):
        if os.path.isdir(os.path.join(package_docs_path, filename)):
            doc_versions.append(filename)
    file_versions = {}
    for filename in os.listdir(eggs_path):
        if filename.startswith(name) and filename.endswith('.tar.gz'):
            version = filename[:-len('.tar.gz')].split('-')[1]
            file_versions[version] = (filename, 'tar.gz')
    # Get a template or page
    if os.path.exists(os.path.join(package_docs_path, 'index.html')):
        page = DreamweaverTemplateInstance(
            filename=os.path.join(package_docs_path, 'index.html')
        )
    else:
        page = DreamweaverTemplateInstance(
            filename=template_path,
        )
    # Generate the index page
    page['doctitle'] = '<title>%s</title>'%name
    page['heading'] = name
    summary = pkg_info['Summary'][0]
    if not summary.endswith('.'):
        summary += '.'
    page['content'] = '<p>'+summary+'</p>'
    page['content'] += (
        '<p>Development status :%s, License: %s, Author: %s</p>'
    )%(
        pkg_info['Status'][0],
        pkg_info['License'][0],
        pkg_info['Author'][0]
    )
    page['content'] += '<ul>'
    doc_versions.sort()
    doc_versions.reverse()
    # Add the list of versions for the package
    for i, version in enumerate(doc_versions):
        page['content'] += '<li>'
        if i==0:
            page['content'] += '<b>Latest - '
        else:
            page['content'] += 'Archive - '
        page['content'] += 'Documentation: <a href="%s/index.html">%s</a> '%(
            version, 
            version,
        )
        if file_versions.has_key(version):
            page['content'] += 'Download: '
            filename, type = file_versions[version]
            page['content'] += '<a href="../eggs/%s">%s</a>, '%(
                filename, 
                type
            )
        page['content'] = page['content'][:-2]
        if i==0:
            page['content'] += '</b>'
        page['content'] += '</li>'
    page['content'] += '</ul>'
    # Save the page
    page.save_as_page(
        filename=os.path.join(package_docs_path, 'index.html'),
    )

def egg_index_page(docs_path, template_path):
    names = []
    for filename in os.listdir(os.path.join(docs_path, 'eggs')):
        if filename.endswith('.tar.gz'):
            names.append(filename)
    if os.path.exists(os.path.join(docs_path, 'eggs', 'index.html')):
        page = DreamweaverTemplateInstance(
            filename=os.path.join(docs_path, 'eggs', 'index.html')
        )
    else:
        page = DreamweaverTemplateInstance(
            filename=template_path
        )
    page['doctitle'] = '<title>%s</title>'%'Eggs'
    page['heading'] = 'Eggs'
    page['content'] = '<ul>'
    names.sort()
    for i, name in enumerate(names):
        page['content'] += '<li>'
        page['content'] +=  '<a href="%s">%s</a>'%(name, name)
    page['content'] += '</ul>'
    page.save_as_page(
        filename=os.path.join(docs_path, 'eggs', 'index.html')
    )

def main_index_page(docs_path, template_path, exclude=None):
    if exclude is None:
        exclude=[]
    # We don't treat the eggs dir as a package
    exclude.append('eggs')
    versions = []
    for filename in os.listdir(docs_path):
        if filename not in exclude and \
           os.path.isdir(os.path.join(docs_path, filename)):
            versions.append(filename)
    if os.path.exists(os.path.join(docs_path, 'index.html')):
        page = DreamweaverTemplateInstance(
            filename=os.path.join(docs_path, 'index.html')
        )
    else:
        page = DreamweaverTemplateInstance(
            filename=template_path
        )
    name = 'Code'
    page['doctitle'] = '<title>%s</title>'%name
    page['heading'] = name
    data = page['content']
    top = data[:data.find('<ul>')]
    last = data[data.find('</ul>')+5:]
    page['content'] = top+'<ul>'
    versions.sort()
    for i, version in enumerate(versions):
        page['content'] += '<li><a href="%s/index.html">%s</a> %s</li>'%(
            version, 
            version, 
            '',
        )
    page['content'] += '</ul>'+last
    page.save_as_page(
        filename=os.path.join(docs_path, 'index.html')
    )

